class MemoOptimizer {
  constructor() {
    this.componentUpdates = new Map();
    this.propChanges = new Map();
  }

  analyze(componentName, prevProps, nextProps) {
    this.trackUpdate(componentName);
    this.trackPropChanges(componentName, prevProps, nextProps);
  }

  trackUpdate(componentName) {
    this.componentUpdates.set(
      componentName,
      (this.componentUpdates.get(componentName) || 0) + 1
    );
  }

  trackPropChanges(componentName, prevProps, nextProps) {
    if (!this.propChanges.has(componentName)) {
      this.propChanges.set(componentName, new Map());
    }

    const propChanges = this.propChanges.get(componentName);
    Object.keys(nextProps).forEach(prop => {
      if (prevProps[prop] !== nextProps[prop]) {
        propChanges.set(prop, (propChanges.get(prop) || 0) + 1);
      }
    });
  }

  getSuggestions() {
    const suggestions = [];
    this.componentUpdates.forEach((updateCount, componentName) => {
      const propChangeFrequency = this.propChanges.get(componentName);
      if (updateCount > 10 && propChangeFrequency) {
        const stableProps = Array.from(propChangeFrequency.entries())
          .filter(([_, changes]) => changes < updateCount * 0.2)
          .map(([prop]) => prop);

        if (stableProps.length > 0) {
          suggestions.push({
            component: componentName,
            suggestion: 'Consider using React.memo',
            reason: `Component rerenders frequently (${updateCount} times) but props ${stableProps.join(', ')} rarely change`,
            stableProps
          });
        }
      }
    });
    return suggestions;
  }
}

module.exports = MemoOptimizer;
